"use strict";

var _fs = _interopRequireDefault(require("fs"));
var _getCredentials = require("@commercetools/get-credentials");
var _pino = _interopRequireDefault(require("pino"));
var _prettyError = _interopRequireDefault(require("pretty-error"));
var _yargs = _interopRequireDefault(require("yargs"));
var _main = _interopRequireDefault(require("./main"));
var _package = require("../package.json");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
process.title = 'custom-objects-exporter';
var args = _yargs["default"].usage("\nUsage: $0 [options]\n".concat(_package.description)).showHelpOnFail(false).option('output', {
  alias: 'o',
  "default": 'stdout',
  describe: 'Path to output file.'
}).coerce('output', function (arg) {
  if (arg !== 'stdout') return _fs["default"].createWriteStream(String(arg));
  return process.stdout;
}).option('apiUrl', {
  "default": 'https://api.europe-west1.gcp.commercetools.com',
  describe: 'The host URL of the HTTP API service.'
}).option('authUrl', {
  "default": 'https://auth.europe-west1.gcp.commercetools.com',
  describe: 'The host URL of the OAuth API service.'
}).option('accessToken', {
  describe: 'CTP client access token.'
}).option('projectKey', {
  alias: 'p',
  describe: 'API project key.',
  demand: true
}).option('where', {
  alias: 'w',
  describe: 'Specify where predicate.'
}).option('logLevel', {
  "default": 'info',
  describe: 'Logging level: error, warn, info or debug.'
}).option('prettyLogs ', {
  describe: 'Pretty print logs to the terminal',
  type: 'boolean'
}).option('logFile', {
  "default": 'custom-objects-export.log',
  describe: 'Path to where to save logs file.',
  type: 'string'
}).argv;

// instantiate logger
var loggerConfig = {
  level: args.logLevel,
  prettyPrint: args.prettyLogs
};

// If the stdout is used for a data output, save all logs to a log file.
// pino writes logs to stdout by default
var logDestination;
if (args.output === process.stdout) logDestination = _fs["default"].createWriteStream(args.logFile);
var logger = (0, _pino["default"])(loggerConfig, logDestination);

// print errors to stderr if we use stdout for data output
// if we save data to output file errors are already logged by pino
var logError = function logError(error) {
  var errorFormatter = new _prettyError["default"]();
  if (args.logLevel === 'debug') process.stderr.write("ERR: ".concat(errorFormatter.render(error)));else process.stderr.write("ERR: ".concat(error.message || error));
};
var errorHandler = function errorHandler(errors) {
  if (Array.isArray(errors)) errors.forEach(logError);else logError(errors);
  process.exitCode = 1;
};
var resolveCredentials = function resolveCredentials(_args) {
  if (_args.accessToken) return Promise.resolve({});
  return (0, _getCredentials.getCredentials)(_args.projectKey);
};

// Register error listener
args.output.on('error', errorHandler);
resolveCredentials(args).then(function (credentials) {
  var apiConfig = {
    host: args.authUrl,
    apiUrl: args.apiUrl,
    projectKey: args.projectKey,
    credentials: credentials
  };
  var exporterOptions = {
    apiConfig: apiConfig,
    accessToken: args.accessToken,
    predicate: args.where,
    logger: {
      error: logger.error.bind(logger),
      warn: logger.warn.bind(logger),
      info: logger.info.bind(logger),
      debug: logger.debug.bind(logger)
    }
  };
  return new _main["default"](exporterOptions);
}).then(function (customObjectsExporter) {
  return customObjectsExporter.run(args.output);
})["catch"](errorHandler);