"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _nodeFetch = _interopRequireDefault(require("node-fetch"));
var _sdkClient = require("@commercetools/sdk-client");
var _apiRequestBuilder = require("@commercetools/api-request-builder");
var _sdkMiddlewareHttp = require("@commercetools/sdk-middleware-http");
var _sdkMiddlewareAuth = require("@commercetools/sdk-middleware-auth");
var _sdkMiddlewareUserAgent = require("@commercetools/sdk-middleware-user-agent");
var _JSONStream = _interopRequireDefault(require("JSONStream"));
var _silentLogger = _interopRequireDefault(require("./utils/silent-logger"));
var _package = _interopRequireDefault(require("../package.json"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _classCallCheck(a, n) { if (!(a instanceof n)) throw new TypeError("Cannot call a class as a function"); }
function _defineProperties(e, r) { for (var t = 0; t < r.length; t++) { var o = r[t]; o.enumerable = o.enumerable || !1, o.configurable = !0, "value" in o && (o.writable = !0), Object.defineProperty(e, _toPropertyKey(o.key), o); } }
function _createClass(e, r, t) { return r && _defineProperties(e.prototype, r), t && _defineProperties(e, t), Object.defineProperty(e, "prototype", { writable: !1 }), e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
var CustomObjectsExporter = exports["default"] = /*#__PURE__*/function () {
  // Set type annotations

  function CustomObjectsExporter(options) {
    _classCallCheck(this, CustomObjectsExporter);
    if (!options.apiConfig) throw new Error('The constructor must be passed an `apiConfig` object');
    this.apiConfig = options.apiConfig;
    this.client = (0, _sdkClient.createClient)({
      middlewares: [(0, _sdkMiddlewareAuth.createAuthMiddlewareWithExistingToken)(options.accessToken ? "Bearer ".concat(options.accessToken) : ''), (0, _sdkMiddlewareAuth.createAuthMiddlewareForClientCredentialsFlow)(_objectSpread(_objectSpread({}, this.apiConfig), {}, {
        fetch: _nodeFetch["default"]
      })), (0, _sdkMiddlewareUserAgent.createUserAgentMiddleware)({
        libraryName: _package["default"].name,
        libraryVersion: _package["default"].version
      }), (0, _sdkMiddlewareHttp.createHttpMiddleware)({
        host: this.apiConfig.apiUrl,
        enableRetry: true,
        fetch: _nodeFetch["default"]
      })]
    });
    this.predicate = options.predicate;
    this.logger = _objectSpread(_objectSpread({}, _silentLogger["default"]), options.logger);
  }
  return _createClass(CustomObjectsExporter, [{
    key: "run",
    value: function run(outputStream) {
      this.logger.info('Starting Export');
      var jsonStream = _JSONStream["default"].stringify();
      jsonStream.pipe(outputStream);
      CustomObjectsExporter.handleOutput(outputStream, jsonStream, this.client, this.apiConfig.projectKey, this.predicate, this.logger);
    }
  }], [{
    key: "handleOutput",
    value: function handleOutput(outputStream, pipeStream, client, projectKey, predicate, logger) {
      CustomObjectsExporter.fetchObjects(pipeStream, client, projectKey, predicate, logger).then(function () {
        logger.info('Export operation completed successfully');
        if (outputStream !== process.stdout) pipeStream.end();
      })["catch"](function (e) {
        outputStream.emit('error', e);
      });
    }
  }, {
    key: "fetchObjects",
    value: function fetchObjects(output, client, projectKey, predicate, logger) {
      var request = CustomObjectsExporter.buildRequest(projectKey, predicate);
      return client.process(request, function (_ref) {
        var statusCode = _ref.statusCode,
          body = _ref.body;
        if (statusCode !== 200) return Promise.reject(new Error("Request returned error ".concat(statusCode)));
        body.results.forEach(function (object) {
          output.write(object);
        });
        logger.debug("Successfully exported ".concat(body.count, " custom %s"), body.count > 1 ? 'objects' : 'object');
        return Promise.resolve();
      }, {
        accumulate: false
      });
    }
  }, {
    key: "buildRequest",
    value: function buildRequest(projectKey, predicate) {
      var uri = CustomObjectsExporter.buildUri(projectKey, predicate);
      return {
        uri: uri,
        method: 'GET'
      };
    }
  }, {
    key: "buildUri",
    value: function buildUri(projectKey, predicate) {
      var service = (0, _apiRequestBuilder.createRequestBuilder)({
        projectKey: projectKey
      }).customObjects;
      if (predicate) service.where(predicate);
      return service.build();
    }
  }]);
}();